
% parameters
runs = 3;       % number of random conformations to be generated
epsilon = 0.001;% error tolerance
match3 = false; % set this to true to match 3 atoms
silent = true;  % set this to false to have an animation of process (press
                % enter to proceed )


% read in the pdb file.
b = pdbRead('backbone_native.pdb');

% extract the region of interest (residue 34 - 46)
index = (b.resnum <= 46) & (b.resnum >= 34);
originalChain = b.coords(index,:);
chain = originalChain;
n = size(chain, 1);


closedLoop = zeros(runs, size(chain,1), size(chain,2));
for i=1:runs
    closedLoop(i,:,:) = loopClosureCCD(chain, epsilon, match3, silent);
    angles(i) = computeAngles(chain);


    fig = figure;
    set(fig, 'name', ['CCD final conformation ' int2str(i)]);
    subplot(2,2,1); plot3(closedLoop(i,:,1), closedLoop(i,:,2), closedLoop(i,:,3), 'r-o', originalChain(:,1), originalChain(:,2), originalChain(:,3), 'g-x');
    subplot(2,2,2); plot(angles(i).bonds); title('Backbone bond outer angles');
    subplot(2,2,3); plot(angles(i).dihedrals); title('Backbone dihedral angles');
    subplot(2,2,4); plot(angles(i).dihedrals(:,1), angles(i).dihedrals(:,2), 'o'); 
    axis([-180 180 -180 180]); title('Ramachandran plot');

end