function result = computeAngles(chain)
% result = computeAngles(chain)
% returns a structure result, containing two arrays.
% result.bonds contains the outer  bond angles for the chain, and
% result.dihedrals contains the dihedral angles.

% compute bond angles and diherals.
v = diff(chain); % row differences give vectors
len = sqrt(dot(v,v,2));
v = v ./ repmat(len,1,3); % normalize rows

nv = size(v,1);
nv1 = nv-1;
nv2=nv-2;

uv = dot(v(1:nv1,:), v(2:nv,:), 2); % cos of angles between adjacent
result.bonds = 180/pi*acos(uv); % bond angles

uprime = v(1:nv2,:) - repmat(uv(1:nv2),1,3).* v(2:nv1,:); % project prev: u onto vperp
uprime = uprime ./ repmat(sqrt(dot(uprime, uprime, 2)), 1,3); % normalize
wprime = v(3:nv,:) - repmat(uv(2:nv1),1,3).* v(2:nv1,:); % project succ: w onto vperp
wprime = wprime ./ repmat(sqrt(dot(wprime, wprime, 2)), 1,3); % normalize
uw = cross(uprime, wprime, 2); 

sn = -sign(dot(uw,v(2:nv1,:),2)) .* sqrt(dot(uw,uw,2)); % sine is magnitude of cross product
cs = -dot(uprime, wprime, 2); % cosine is dot product
%disp( sn.^2+cs.^2); % check
result.dihedrals = 180/pi*atan2(sn, cs); % compute angles

result.bonds = reshape([NaN; result.bonds; NaN], 3, ceil(nv/3))';
result.dihedrals = reshape([NaN; result.dihedrals; NaN; NaN], 3, ceil(nv/3))';
result.dihedrals(:,3) = 180 + result.dihedrals(:,3) - 360*(result.dihedrals(:,3)>0)