function result = loopClosureCCD(chain, epsilon, match3, silent)
% result = loopClosureCCD(chain, epsilon, match3, silent)
% resturns the loop closure of a random conformation to match
% the first three atoms of chain and the last atom of chain.
% The random conformation is produced by uniformely selecting
% all the dihedral angles in the interval [0,2Pi].
%
% epsilon is the error tolerance allowed.
%
% match3 is an optional boolean parameter if three atoms should
%        be targeted instead of three. The default is false.
%
% silent is an optional boolean paramter indicating if intermediate
%        output should be made. Note that you have to press enter
%        to proceed in every iteration.


if (nargin <= 3) % verbose is default behaviour.
    silent = false;
end

n = size(chain, 1);
anchor = chain(1:3,:);
% decide how many points to match
if (match3)
    target = chain(n-2:n,:);
else
    target = chain(n,:);
end
originalChain = chain;



% construct random cyclic order for the n-5 dihedral degrees
% of freedom.
cycle = rand(1,n-3);
[dump, index] = sort(cycle);
cycle = [1:n-3];
cycle = cycle(index);
size(cycle);


% generate random conformation
chain = randomDihedral(chain, 1, n-3);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plotting animation and output
if (~silent)
    figure;
    plot3(chain(:,1), chain(:,2), chain(:,3), '-o', originalChain(:,1), originalChain(:,2), originalChain(:,3), '-x');
    pause;
    camup('manual');
    campos('manual');
    camva('manual');
    camtarget('manual');
    axis('manual');
    ctarget = camtarget;
    cva = camva;
    cpos = campos;
    cup = camup;
    cproj = camproj;
end
%end animation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% go through dihedral angles until error is small enough
index = 1;
if (epsilon <= 0)
    epsilon = 0.1;
end
error = epsilon + 1;

counter = 0;

while (error > epsilon)
counter = counter + 1;
    
    start = cycle(index);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plotting animation and output
    if (~silent)
        h = plot3(chain(:,1), chain(:,2), chain(:,3),...
            '-o', originalChain(:,1), originalChain(:,2), originalChain(:,3), '-x',...
            chain(start+1:start+2,1), chain(start+1:start+2,2), chain(start+1:start+2,3), 'r-o');

        camtarget(ctarget);
        camva(cva);
        campos(cpos);
        camup(cup);
        camproj(cproj);

        %pause(0.0001);
        pause;

        ctarget = camtarget;
        cva = camva;
        cpos = campos;
        cup = camup;
        cproj = camproj;

        error
    end
    %end animation
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    [chain(start:n,:), error] = modifyDihedral(chain(start:n,:), target);
   
    
    if (index >= n-5)
       index = 1;
     else
       index = index + 1;
    end 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plotting animation and output
if (~silent)
    counter

plot3(chain(:,1), chain(:,2), chain(:,3), '-o', originalChain(:,1), originalChain(:,2), originalChain(:,3), '-x');

end
%end animation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
result = chain;
