function modifiedChain = randomDihedral(chain, startIndex, endIndex)
% modifiedChain = randomDihedral(chain, startIndex, endIndex)
% produces random dihedral angles for a chain of atoms between
% startIndex and endIndex. Note that startIndex = i means that
% the bond between atom i+1 and i+2 is the first to be modified.
% endIndex = j means that the the bond between j+1 and j+2 is the
% last one to be modified. Therefore, endIndex must be in the
% interval [1,n-3], and must be greater or equal than startIndex.

n = size(chain, 1);

if (endIndex <= n-3) && (startIndex <= endIndex) && (startIndex > 0) && (endIndex > 0)
    for i = startIndex:endIndex
        randomAngle = rand(1,1) * 2 * pi;
        chain(i:n,:) = rotateDihedral(randomAngle, chain(i:n,:));
    end
end

modifiedChain = chain;