function modifiedChain = rotateDihedral(rotationAngle, chain)
% modifiedChain = rotateDihedral(rotationAngle, chain)
% returns the modified chain in which the dihedral angle defined by
% the first four atoms in the chain is rotated by rotationAngle
% around the vector chain(3,:)-chain(2,:).


% construct rotation quaternion
n = size(chain, 1);
sinTemp = sin(rotationAngle/2);
rotationAxis = chain(3,:) - chain(2,:);
rotationAxis = rotationAxis ./ norm(rotationAxis);
rotationQuaternion = [cos(rotationAngle/2), sinTemp * rotationAxis];

% translate points, rotationAxis through origin
basePoint = chain(3,:);
translation = repmat(basePoint,n-3,1);
tempChain = chain(4:n,:) - translation;

% do rotation
tempResult = rotQ(tempChain, conjQ(rotationQuaternion));

% translate points again, return result
modifiedChain = [chain(1:3,:); tempResult + translation];


